const express = require("express");
const router = express.Router();
const DecryptWithServerKey = require("./../../services/decryptionservice");
const DecryptWithStellarSeed = require("./../../services/seedDecryptionService");
const Patient = require("./../../model/patient/patient");
const axios = require("axios");
const StellarSDK = require("stellar-sdk");
const htmlpdf = require("node-htmlpdf");
const IPFS = require("ipfs-infura");
const fs = require("fs");
const request = require("request");
const RSAEncrypt = require("./../../services/rsaEncoder");
const GenerateAESKey = require("./../../services/aesKeyGenerator");
const EncryptionService = require("./../../services/encryptionservice");
const AssetKey = require("./../../model/stellar/assetkeys");
const CreateTrustline = require("./../../services/trustlineCreator");
const GetIssuerPair = require("./../../services/getIssuerKeyPairFromDB");
const DoManageData = require("./../../services/manageDataForAsset");
const SendPayment = require("./../../services/paymentOperation");
const UpdateNFTRequestStatus = require("./../../services/updateNFTRequest");
let currentHash;
let genesisHash;
let isGenesisPassed;
let htmlString;

router.post("/", async (req, res) => {
  //Get the payload and decrypt
  const decryptedPatientKey = req.body.patientkey;
  const decryptedPatientSeed = req.body.patientseed;
  const decryptedPractitionerKey = req.body.practitioner;
  const decryptedPractitionerRsaKey = req.body.practitionerrsa;
  const decryptedAssetCode = req.body.assetcode;
  const b64DecodedRsa = Buffer.from(
    decryptedPractitionerRsaKey,
    "base64"
  ).toString("utf-8");

  //get the current hash from DB
  try {
    let patientDetails = await Patient.findOne({
      publickey: decryptedPatientKey,
    });

    if (patientDetails == null) {
      console.log("No patient details found for : " + decryptedPatientKey);
      console.log("/patient/createnft - NFT creation and sharing failed");
      return res
        .status(404)
        .json({ message: "Patient not found for : " + decryptedPatientKey });
    } else {
      //take the current hash and thegenesis hash recorded
      let initialCurrentHash = patientDetails.currenthash;
      genesisHash = patientDetails.genesishash;

      //check if this is the genesis hash
      if (genesisHash === initialCurrentHash) {
        //send response with the error message
        console.log(
          "This user does not have enough information to create an EHR"
        );
        console.log("/patient/createnft - NFT creation and sharing failed");
        return res.status(404).json({
          message:
            "This user does not have enough information to create an EHR",
        });
      } else {
        htmlString = `<!DOCTYPE html> 
        <html lang="en">
          <head>
            <meta charset="UTF-8" />
            <meta http-equiv="X-UA-Compatible" content="IE=edge" />
            <meta name="viewport" content="width=device-width, initial-scale=1.0" />
          </head>
          <body>
            <center>
                <h1>Electronic Health Record</h1>
                <h2>User Details</h1>    
            </center>
            <p style="float: inline-start; font-weight: 300; font-size: smaller;">*Please note that this is a computer generated health record</p>`;
        let timestamp = new Date().toString();
        let timestampString =
          `<p style="float: inline-start; font-weight: 300; font-size: smaller;">Generated time : ` +
          timestamp +
          `</p><hr />`;
        htmlString = htmlString + timestampString;
        //get the genesis block details
        try {
          console.log(
            "Calling the genesis block: " +
              process.env.STELLARTESTNET +
              "/transactions/" +
              genesisHash.toString()
          );
          const genesisTransactionDetails = await axios.default.get(
            process.env.STELLARTESTNET +
              "/transactions/" +
              genesisHash.toString()
          );

          if (genesisTransactionDetails.status != 200) {
            console.log(
              "Error when getting genesis transaction : Code - " +
                genesisTransactionDetails.status +
                " Status : " +
                genesisTransactionDetails.statusText
            );
            console.log("/patient/createnft - NFT creation and sharing failed");
            return res.status(500).json({
              message:
                "Error when getting genesis transaction : Code - " +
                genesisTransactionDetails.status +
                " Status : " +
                genesisTransactionDetails.statusText,
            });
          } else {
            const genesisOperations = JSON.stringify(
              StellarSDK.xdr.TransactionEnvelope.fromXDR(
                genesisTransactionDetails.data.envelope_xdr,
                "base64"
              )
                .v1()
                .tx()
                .operations()
            );

            const genesisJsonOb = JSON.parse(genesisOperations);

            //loop the operations
            for (let i = 0; i < genesisJsonOb.length; i++) {
              let name = Buffer.from(
                genesisJsonOb[i]._attributes.body._value._attributes.dataName
                  .data
              );
              let value = Buffer.from(
                genesisJsonOb[i]._attributes.body._value._attributes.dataValue
                  .data
              );
              //append genesis details to the HTML string
              htmlString =
                htmlString +
                `<p>` +
                name.toString() +
                ` : ` +
                DecryptWithStellarSeed(value.toString(), decryptedPatientSeed) +
                `</p>`;
            }
          }
        } catch (errorWhenCallingGenesisTxn) {
          console.log(
            "Error when getting genesis transaction details from stellar : " +
              errorWhenCallingGenesisTxn
          );
          console.log("/patient/createnft - NFT creation and sharing failed");
          return res.status(500).json({
            message:
              "Error when getting genesis transaction details from stellar : " +
              errorWhenCallingGenesisTxn,
          });
        }

        currentHash = initialCurrentHash;
        htmlString =
          htmlString +
          `<hr> 
        <br>
        <h3>Recorded mental health statuses and emotional statuses according to previous chat bot interaction</h3>
        <br>`;
        //loop through all the previous transactions
        do {
          //get the details by calling the endpoints get transaction details
          try {
            console.log(
              "Calling : " +
                process.env.STELLARTESTNET +
                "/transactions/" +
                currentHash.toString()
            );
            //call the stellar endpoint
            const transactionDetails = await axios.default.get(
              process.env.STELLARTESTNET +
                "/transactions/" +
                currentHash.toString()
            );
            //check the status of the transaction
            if (transactionDetails.status != 200) {
              console.log(
                "Error when getting transaction : Code - " +
                  transactionDetails.status +
                  " Status : " +
                  transactionDetails.statusText
              );
              console.log(
                "/patient/createnft - NFT creation and sharing failed"
              );
              return res.status(500).json({
                message:
                  "Error when getting transaction : Code - " +
                  transactionDetails.status +
                  " Status : " +
                  transactionDetails.statusText,
              });
            } else {
              const operations = JSON.stringify(
                StellarSDK.xdr.TransactionEnvelope.fromXDR(
                  transactionDetails.data.envelope_xdr,
                  "base64"
                )
                  .v1()
                  .tx()
                  .operations()
              );

              const jsonOb = JSON.parse(operations);

              //loop the operations
              for (let i = 0; i < jsonOb.length; i++) {
                let name = Buffer.from(
                  jsonOb[i]._attributes.body._value._attributes.dataName.data
                );
                let value = Buffer.from(
                  jsonOb[i]._attributes.body._value._attributes.dataValue.data
                );

                //check the name type
                if (name.toString() === "Timestamp") {
                  //append to HTML string
                  htmlString =
                    htmlString +
                    `<p>Date and time : ` +
                    value.toString() +
                    `</p>`;
                } else if (
                  name.toString() === "Emotion" ||
                  name.toString() === "Mental state"
                ) {
                  //append to HTML string
                  htmlString =
                    htmlString +
                    `<p>` +
                    name.toString() +
                    ` : ` +
                    DecryptWithStellarSeed(
                      value.toString(),
                      decryptedPatientSeed
                    ) +
                    `</p>`;
                }
              }
              htmlString = htmlString + `<br><br>`;
              //Assign the current hash
              currentHash = Buffer.from(
                jsonOb[0]._attributes.body._value._attributes.dataValue.data
              ).toString();

              //skipping the genesis hash
              if (currentHash === genesisHash) {
                isGenesisPassed = true;
              } else {
                isGenesisPassed = false;
              }
            }
          } catch (errorWhenGettingTxnFromStellar) {
            console.log(
              "Error when getting transaction details from stellar : " +
                errorWhenGettingTxnFromStellar
            );
            console.log("/patient/createnft - NFT creation and sharing failed");

            return res.status(500).json({
              message:
                "Error when getting transaction details from stellar : " +
                errorWhenGettingTxnFromStellar,
            });
          }
        } while (!isGenesisPassed);
        console.log("All the patient data taken from the blockchain");
        htmlString = htmlString + `</body></html>`;
        console.log("HTML code for EHR generated");

        //create the IPFS instance
        const ipfs = new IPFS({
          host: process.env.IPFSHOST,
          port: process.env.IPFSPORT,
          protocol: process.env.IPFSPROTOCOL,
          projectId: process.env.IPFSPROJECTID,
          projectSecret: process.env.IPFSAPIKEY,
        });

        //generate AES key for EHR encryption
        const aesKey = GenerateAESKey();

        //encrypt HTML string using generated AES key
        const encryptedHTML = EncryptionService.EncryptWithStellarSeed(
          htmlString,
          aesKey
        );

        console.log(
          "---------------------------------Encrypted EHR------------------------------"
        );
        console.log(encryptedHTML);
        console.log(
          "----------------------------------------------------------------------------"
        );

        //encode the AES key using practitioner RSA key
        const encodedAesKey = RSAEncrypt(b64DecodedRsa, aesKey);

        console.log("AES key (encoded) ", encodedAesKey);

        //add AES key and Asset code to DB
        const assetObj = new AssetKey({
          assetcode: decryptedAssetCode,
          key: encodedAesKey,
        });

        try {
          const saveInAssetKeys = await assetObj.save();

          console.log("Asset code and Key saved in DB : ", saveInAssetKeys);

          //add the EHR content to IPFS and get the hash
          ipfs
            .add(encryptedHTML.toString())
            .then(async (response) => {
              let ipfsContentCID = response.toString();
              //!Use this when getting EHR
              // request.post(
              //   {
              //     url: "https://ipfs.infura.io:5001/api/v0/cat",
              //     qs: {
              //       arg: ipfsContentCID,
              //     },
              //     headers: {
              //       Authorization:
              //         "Basic " +
              //         Buffer.from(
              //           process.env.IPFSPROJECTID + ":" + process.env.IPFSAPIKEY
              //         ).toString("base64"),
              //     },
              //   },
              //   (error, response, body) => {
              //     if (error) {
              //       console.log(error);
              //     } else {
              //       //create the PDF document for EHR
              //       let path =
              //         `./patientehr/` + decryptedPatientKey + `-EHR.pdf`;
              //       let pdfOptions = {
              //         path: path,
              //         format: "A4",
              //       };
              //       htmlpdf
              //         .create(htmlString, pdfOptions)
              //         .then((buff) => {
              //           console.log("PDF created");
              //         })
              //         .catch((errorWhenGeneratingPDF) => {
              //           console.log(
              //             "Error when generating PDF : " +
              //               errorWhenGeneratingPDF
              //           );
              //           console.log(
              //             "/patient/createnft - NFT creation and sharing failed"
              //           );
              //           return res.status(500).json({
              //             message:
              //               "Error when generating PDF : " +
              //               errorWhenGeneratingPDF,
              //           });
              //         });
              //     }
              //   }
              // );

              //get issuer key pair from DB
              const issuerKeyPair = await GetIssuerPair(
                decryptedPatientKey,
                "patients"
              );

              //create the trustline with the issuer account from patient
              CreateTrustline(
                decryptedAssetCode,
                DecryptWithServerKey(issuerKeyPair.issuerKey),
                decryptedPatientSeed
              )
                .then((result) => {
                  //Manage data operation from issuer account
                  DoManageData(
                    DecryptWithServerKey(issuerKeyPair.issuerSeed),
                    decryptedAssetCode,
                    ipfsContentCID
                  )
                    .then((success) => {
                      if (success) {
                        console.log("NFT manage data submitted from issuer ");

                        //payment operation to transfer asset from issuer to patient
                        SendPayment(
                          decryptedAssetCode,
                          DecryptWithServerKey(issuerKeyPair.issuerSeed),
                          decryptedPatientKey,
                          DecryptWithServerKey(issuerKeyPair.issuerKey)
                        )
                          .then((success) => {
                            if (success) {
                              console.log(
                                "Payment operation from issuer to patient success"
                              );

                              //payment operation to transfer asset from patient to issuer
                              SendPayment(
                                decryptedAssetCode,
                                decryptedPatientSeed,
                                decryptedPractitionerKey,
                                DecryptWithServerKey(issuerKeyPair.issuerKey)
                              )
                                .then((success) => {
                                  if (success) {
                                    console.log(
                                      "Payment operation from patient to practitioner success"
                                    );

                                    //Update the NFT req collection
                                    UpdateNFTRequestStatus(decryptedAssetCode)
                                      .then((success) => {
                                        if (success) {
                                          res.status(200).json({
                                            message:
                                              "NFT EHR issued successfully",
                                          });
                                          console.log(
                                            "/patient/createnft - NFT creation and sharing success"
                                          );
                                        } else {
                                          console.log(
                                            "NFT Request update failed "
                                          );
                                          console.log(
                                            "/patient/createnft - NFT creation and sharing failed"
                                          );
                                          return res.status(500).json({
                                            message:
                                              "NFT Request update failed ",
                                          });
                                        }
                                      })
                                      .catch((errorWhenUpdatingCollection) => {
                                        console.log(
                                          "Error updating NFT request status : ",
                                          errorWhenUpdatingCollection
                                        );
                                        console.log(
                                          "/patient/createnft - NFT creation and sharing failed"
                                        );
                                        return res.status(500).json({
                                          message:
                                            "Error updating NFT request status : ",
                                          errorWhenUpdatingCollection,
                                        });
                                      });
                                  } else {
                                    console.log(
                                      "NFT payment operation failed from patient to practitioner"
                                    );
                                    console.log(
                                      "/patient/createnft - NFT creation and sharing failed"
                                    );
                                    return res.status(500).json({
                                      message:
                                        "NFT payment operation failed from patient to practitioner",
                                    });
                                  }
                                })
                                .catch(
                                  (
                                    errorWhenSendingAssetFromPatientToPractitioner
                                  ) => {
                                    console.log(
                                      "Error when sending NFT from patient to practitioner : ",
                                      errorWhenSendingAssetFromPatientToPractitioner
                                    );
                                    console.log(
                                      "/patient/createnft - NFT creation and sharing failed"
                                    );
                                    return res.status(500).json({
                                      message:
                                        "Error when sending NFT from patient to practitioner : ",
                                      errorWhenSendingAssetFromPatientToPractitioner,
                                    });
                                  }
                                );
                            } else {
                              console.log(
                                "NFT payment operation failed from issuer to patient"
                              );
                              console.log(
                                "/patient/createnft - NFT creation and sharing failed"
                              );
                              return res.status(500).json({
                                message:
                                  "NFT payment operation failed from issuer to patient",
                              });
                            }
                          })
                          .catch((errorWhenSendingAssetToPatient) => {
                            console.log(
                              "Error when submitting issuer to patient manage data : " +
                                errorWhenSendingAssetToPatient
                            );
                            console.log(
                              "/patient/createnft - NFT creation and sharing failed"
                            );
                            return res.status(500).json({
                              message:
                                "Error when submitting issuer to patient manage data  : " +
                                errorWhenSendingAssetToPatient,
                            });
                          });
                      } else {
                        console.log(
                          "NFT manage data not submitted from issuer "
                        );
                        console.log(
                          "/patient/createnft - NFT creation and sharing failed"
                        );
                        return res.status(500).json({
                          message: "NFT manage data not submitted from issuer ",
                        });
                      }
                    })
                    .catch((errorWhenSubmittingIssuerManageData) => {
                      console.log(
                        "Error when submitting issuer manage data : " +
                          errorWhenSubmittingIssuerManageData
                      );
                      console.log(
                        "/patient/createnft - NFT creation and sharing failed"
                      );
                      return res.status(500).json({
                        message:
                          "Error when submitting issuer manage data  : " +
                          errorWhenSubmittingIssuerManageData,
                      });
                    });
                })
                .catch((errorWhenCreatingPatientTrustLine) => {
                  console.log(
                    "Error when creating the patient - issuer trustline : " +
                      errorWhenCreatingPatientTrustLine
                  );
                  console.log(
                    "/patient/createnft - NFT creation and sharing failed"
                  );
                  return res.status(500).json({
                    message:
                      "Error when creating the patient - issuer trustline : " +
                      errorWhenCreatingPatientTrustLine,
                  });
                });

              // //load the patient account
              // const userSourceKeyPair =
              //   StellarSDK.Keypair.fromSecret(decryptedPatientSeed);
              // const userPublicKey = userSourceKeyPair.publicKey();
              // const server = new StellarSDK.Server(process.env.STELLARTESTNET);
              // const account = await server.loadAccount(userPublicKey);
              // const fee = await server.fetchBaseFee();

              // const issueNFTTxn = new StellarSDK.TransactionBuilder(account, {
              //   fee,
              //   networkPassphrase: StellarSDK.Networks.TESTNET,
              // })
              //   .addMemo(StellarSDK.Memo.text("Issue NFT"))
              //   .addOperation(
              //     StellarSDK.Operation.payment({
              //       asset: new StellarSDK.Asset("NFTEHR", decryptedPatientKey),
              //       amount: "1",
              //       destination: decryptedPractitionerKey,
              //     })
              //   )
              //   .addOperation(
              //     StellarSDK.Operation.manageData({
              //       name: "NFTEHR",
              //       value: ipfsContentCID,
              //     })
              //   )
              //   .setTimeout(30)
              //   .build();

              // //sign transaction
              // issueNFTTxn.sign(userSourceKeyPair);

              // //send transaction to blockchain
              // try {
              //   const transactionResult = await server.submitTransaction(
              //     issueNFTTxn
              //   );
              //   console.log("NFT issued : HASH : " + transactionResult.hash);
              //   console.log(
              //     "View NFT issued transaction at : https://horizon-testnet.stellar.org/transactions/" +
              //       transactionResult.hash
              //   );
              //   res.status(200).json({
              //     message: "NFT EHR issued successfully",
              //     hash: transactionResult.hash,
              //   });
              //   console.log(
              //     "/patient/createnft - NFT creation and sharing success"
              //   );
              // } catch (errorWhenIssuingNFT) {
              //   console.log("Error when issuing NFT : " + errorWhenIssuingNFT);
              //   console.log(
              //     "/patient/createnft - NFT creation and sharing failed"
              //   );
              //   return res.status(500).json({
              //     message: "Error when issuing NFT : " + errorWhenIssuingNFT,
              //   });
              // }
            })
            .catch((errorWhenAddingToIPFS) => {
              console.log(
                "Error when adding records to IPFS : " + errorWhenAddingToIPFS
              );
              console.log(
                "/patient/createnft - NFT creation and sharing failed"
              );
              return res.status(500).json({
                message:
                  "Error when adding records to IPFS : " +
                  errorWhenAddingToIPFS,
              });
            });
        } catch (errorWhenInsertingAssetCodeAndKey) {
          console.log(
            "Error when adding asset code to DB: " +
              errorWhenInsertingAssetCodeAndKey
          );
          console.log("/patient/createnft - NFT creation and sharing failed");
          return res.status(500).json({
            message:
              "Error when adding asset code to DB: " +
              errorWhenInsertingAssetCodeAndKey,
          });
        }
      }
    }
  } catch (errorWhenGettingCurrentHash) {
    console.log(
      "Error when getting user details from DB: " + errorWhenGettingCurrentHash
    );
    console.log("/patient/createnft - NFT creation and sharing failed");
    return res.status(500).json({
      message:
        "Error when getting user details from DB: " +
        errorWhenGettingCurrentHash,
    });
  }
});

module.exports = router;
