const { MongoClient } = require("mongodb");
require("dotenv").config();

const url = process.env.DATABASE_URL;

async function generateRandomAssetCode() {
  const client = await MongoClient.connect(url, {
    useNewUrlParser: true,
    useUnifiedTopology: true,
  });

  try {
    const db = client.db();
    const collection = db.collection("assetcodes");

    let assetCode = generateCode();

    // Check if the generated asset code is already in the database
    let found = await isAssetCodeInDatabase(collection, assetCode);

    while (found) {
      // If so, generate a new random asset code
      assetCode = generateCode();
      found = await isAssetCodeInDatabase(collection, assetCode);
    }

    // At this point, assetCode is guaranteed to be a unique asset code
    // and can be used for further processing

    // Insert the new asset code into the collection
    const result = await collection.insertOne({ code: assetCode });
    return assetCode;
  } finally {
    // Close the client connection after the query has completed
    client.close();
  }
}

function generateCode() {
  let result = "";
  const characters =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
  const charactersLength = characters.length;
  for (let i = 0; i < 12; i++) {
    result += characters.charAt(Math.floor(Math.random() * charactersLength));
  }
  return result;
}

function isAssetCodeInDatabase(collection, assetCode) {
  return new Promise((resolve, reject) => {
    collection.findOne({ code: assetCode }, (err, result) => {
      if (err) {
        reject(err);
      } else {
        resolve(result !== null);
      }
    });
  });
}

module.exports = generateRandomAssetCode;
