const StellarSdk = require("stellar-sdk");
const server = new StellarSdk.Server(process.env.STELLARTESTNET);

async function CreateTrustline(assetCode, issuerKey, sourceAccountSeed) {
  //load user account
  const userSourceKeyPair = StellarSdk.Keypair.fromSecret(sourceAccountSeed);
  const userPublicKey = userSourceKeyPair.publicKey();
  const account = await server.loadAccount(userPublicKey);
  const fee = await server.fetchBaseFee();

  //create trustline transaction
  const trustlineTxn = new StellarSdk.TransactionBuilder(account, {
    fee,
    networkPassphrase: StellarSdk.Networks.TESTNET,
  })
    .addMemo(StellarSdk.Memo.text("Trustline creation"))
    .addOperation(
      StellarSdk.Operation.changeTrust({
        asset: new StellarSdk.Asset(assetCode, issuerKey),
        limit: "0.0000001",
      })
    )
    .setTimeout(30)
    .build();

  trustlineTxn.sign(userSourceKeyPair);

  try {
    const transactionResult = await server.submitTransaction(trustlineTxn);
    console.log(
      `Transaction hash for issuer trust line : ${transactionResult.hash}`
    );
    return true;
  } catch (error) {
    console.error("Error creating trustline:", error);
    return false;
  }
}
module.exports = CreateTrustline;
