import { useEffect, useState } from "react";
import { LOCALBACKEND } from "../../env";
import { Button, StyleSheet, View, Text } from "react-native";
import AsyncStorage from "@react-native-async-storage/async-storage";

export default function PatientRequests() {
  const [data, setData] = useState([]);

  useEffect(() => {
    fetchRequests();
  }, []);

  const fetchRequests = async () => {
    //get key from async storage
    try {
      const practitionerKey = await AsyncStorage.getItem(
        "practitionerStellarPublicKey"
      );
      try {
        const requestUrl =
          LOCALBACKEND +
          `/practitioner/practitionerrequests/${practitionerKey}`;

        console.log(requestUrl);

        const response = await fetch(requestUrl);
        const json = await response.json();
        setData(json.data);
      } catch (errorFetchingRequests) {
        console.log(
          "Error when fetching request data : ",
          errorFetchingRequests
        );
      }
    } catch (errorWhenGettingKey) {
      console.log(
        "Error when getting the public key from the async storage : ",
        errorWhenGettingKey
      );
    }
  };

  const renderItem = (item) => {
    return (
      <View key={item._id}>
        <Text>Key: {item.patient}</Text>
        <Button title="Accept" onPress={() => handleAcceptance(item)}></Button>
        <Button title="Reject" onPress={() => handleDecline(item)}></Button>
      </View>
    );
  };

  const handleAcceptance = async (item) => {
    console.log("Accepted");
  };

  const handleDecline = async (item) => {
    console.log("Rejected");
  };

  return (
    <View style={styles.container}>{data.map((item) => renderItem(item))}</View>
  );
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
    justifyContent: "center",
    alignItems: "center",
  },
  text: {
    fontSize: 24,
    fontWeight: "bold",
  },
});
